import json
import logging

from django.http import HttpResponse, HttpResponseForbidden
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View

from ..models import AdyenNotification
from .hmac import create_hmac

logger = logging.getLogger(__name__)


class NotificationView(View):
    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        return super(NotificationView, self).dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        """
        Note that Adyen requires SSL/TLS client-certificate validation
        but we don't do that. We do validate that the notification originates
        from Adyen by verifying the HMAC.

        TODO: Validate SSL/TLS client-certificate validation.
        """
        logger.debug(_('New notification'))

        # After we verify that the notification originates from Adyen,
        # we store it in our database.

        hmac_value = create_hmac(request.POST)

        if ('additionalData.hmacSignature' not in request.POST or
                hmac_value != request.POST['additionalData.hmacSignature']):
            return HttpResponseForbidden()

        # TODO Validate request.POST. External data should really be always
        # be validated.

        json_params = json.dumps(request.POST)

        notification = AdyenNotification.objects.create(notification=json_params)
        logger.debug(_('Notification saved | id: %s'), notification.id)

        return HttpResponse('[accepted]')



import base64
import binascii
import hashlib
import hmac
import logging
from collections import OrderedDict
from datetime import timedelta

from django.utils import timezone

from djadyen import settings

logger = logging.getLogger('adyen')


def escape_val(val):
    try:
        return val.replace('\\', '\\\\').replace(':', '\\:')
    except AttributeError:
        return str(val)


def sign_params(params):
    """
    Description of how the HMAC can be calculated can be found here

    https://docs.adyen.com/developers/classic-integration/hosted-payment-pages/hmac-signature-calculation
    """
    hmac_key = binascii.a2b_hex(settings.ADYEN_MERCHANT_SECRET)

    params = OrderedDict(sorted(params.items(), key=lambda t: t[0]))
    logger.debug("Ordered Params: %s", params)

    signing_string = ':'.join(map(escape_val, list(params.keys()) + list(params.values())))
    logger.debug("Signing Params: %s", signing_string)

    hmac_string = hmac.new(hmac_key, signing_string.encode('utf-8'), hashlib.sha256)
    logger.debug("HMAC: %s", hmac_string)

    params['merchantSig'] = base64.b64encode(hmac_string.digest()).decode("utf-8")
    logger.debug("merchantSig: %s", params['merchantSig'])
    return params


class HPPPaymenRequest:
    def __init__(self, merchant_reference, payment_amount, result_url,
                 shopper_email=None, brand_code=None, issuer_id=None):
        self.merchant_reference = merchant_reference
        self.payment_amount = payment_amount
        self.result_url = result_url

        self.shopper_email = shopper_email
        self.shopper_locale = 'nl_NL'
        self.brand_code = brand_code
        self.issuer_id = issuer_id
        self.skin_code = settings.ADYEN_SKIN_CODE
        self.currency_code = settings.ADYEN_CURRENCYCODE
        self.merchant_account = settings.ADYEN_MERCHANT_ACCOUNT

        self.ship_before_date = timezone.now()

        self.session_validity = (
            timezone.now() + timedelta(minutes=settings.ADYEN_SESSION_MINUTES)
        )

    def get_adyen_url(self):
        page = 'hpp/select.shtml'
        if self.brand_code:
            page = 'hpp/details.shtml'
        if self.issuer_id:
            page = 'hpp/skipDetails.shtml'
        url = '{}{}'.format(settings.ADYEN_URL, page)

        logger.debug("Adyen Url: %s", url)
        return url

    @classmethod
    def from_object(cls, obj, return_url):
        kwargs = {
            'merchant_reference': obj.reference,
            'payment_amount': obj.get_price_in_cents(),
            'result_url': '{}{}'.format(settings.ADYEN_HOST_URL, return_url)
        }
        if obj.email:
            kwargs['shopper_email'] = obj.email
        if obj.payment_option:
            kwargs['brand_code'] = obj.payment_option.adyen_name
        if obj.issuer:
            kwargs['issuer_id'] = obj.issuer.adyen_id

        return cls(**kwargs)

    def get_adyen_params(self):
        params = {
            'shipBeforeDate': self.ship_before_date.isoformat(),
            'shopperLocale': self.shopper_locale,
            'merchantReference': self.merchant_reference,
            'paymentAmount': self.payment_amount,
            'resURL': self.result_url,
            'currencyCode': self.currency_code,
            'merchantAccount': self.merchant_account,
            'shopperEmail': self.shopper_email,
            'skinCode': self.skin_code,
            'sessionValidity': self.session_validity.isoformat(),
        }

        if self.issuer_id:
            params['issuerId'] = self.issuer_id

        if self.brand_code is not None:
            params['brandCode'] = self.brand_code

        return sign_params(params)


class HPPPaymentResponse:
    def __init__(self, auth_result, psp_reference, merchant_reference,
                 skin_code, payment_method, shopper_locale, merchant_return_data, reason, merchant_sig):
        self.auth_result = auth_result
        self.psp_reference = psp_reference
        self.merchant_reference = merchant_reference
        self.skin_code = skin_code
        self.payment_method = payment_method
        self.shopper_locale = shopper_locale
        self.merchant_return_data = merchant_return_data
        self.reason = reason
        self.merchant_sig = merchant_sig

    @classmethod
    def from_data(cls, data):
        kwargs = {
            'auth_result': data.get('authResult'),
            'psp_reference': data.get('pspReference'),
            'merchant_reference': data.get('merchantReference'),
            'skin_code': data.get('skinCode'),
            'payment_method': data.get('paymentMethod'),
            'shopper_locale': data.get('shopperLocale'),
            'merchant_return_data': data.get('merchantReturnData'),
            'reason': data.get('reason'),
            'merchant_sig': data.get('merchantSig'),
        }

        return cls(**kwargs)

    def get_adyen_params(self):
        params = {
            'authResult': self.auth_result,
            'merchantReference': self.merchant_reference,
            'skinCode': self.skin_code,
            'paymentMethod': self.payment_method,
            'shopperLocale': self.shopper_locale,
        }

        if self.psp_reference:
            params['pspReference'] = self.psp_reference
        if self.merchant_return_data:
            params['merchantReturnData'] = self.merchant_return_data
        if self.reason:
            params['reason'] = self.reason

        return sign_params(params)

    def is_valid(self):
        if self.merchant_sig is None:
            return False

        calculated_params = self.get_adyen_params()
        return calculated_params['merchantSig'] == self.merchant_sig



import logging

from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.views.generic import FormView
from django.views.generic.detail import SingleObjectMixin

from djadyen import settings

from .choices import Status
from .forms import PaymentForm
from .hpp import HPPPaymenRequest, HPPPaymentResponse, sign_params

try:
    from urllib.parse import urlencode
except Exception:
    from urllib import urlencode


logger = logging.getLogger('adyen')


class AdyenRedirectView(SingleObjectMixin, FormView):
    """
    A view which initiates the Adyen payment by redirecting the user to Adyen.

    This view renders a form with the form-data which Adyen accepts, which points
    to an URL which Adyen accepts.

    For testing purposes you can set 'ADYEN_ENABLED' to False, which instead of
    posting the data to Adyen, posts the data to this view, which then processes
    it in similar fashion as Adyen.
    """

    template_name = 'adyen/form.html'
    form_class = PaymentForm
    slug_field = 'reference'
    slug_url_kwarg = 'reference'

    def get_form_kwargs(self):
        kwargs = super(AdyenRedirectView, self).get_form_kwargs()
        kwargs.update({
            "initial": self.payment_request.get_adyen_params()
        })
        return kwargs

    def get_context_data(self, **kwargs):
        context = super(AdyenRedirectView, self).get_context_data(**kwargs)
        if settings.ADYEN_ENABLED:
            context.update({
                'adyen_url': self.payment_request.get_adyen_url(),
                'add_csrf': False
            })
        else:
            context.update({
                'adyen_url': self.get_current_url(),
                'add_csrf': True
            })
        return context

    def get_payment_request(self, obj):
        return HPPPaymenRequest.from_object(obj, self.get_next_url())

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        self.payment_request = self.get_payment_request(self.object)

        if self.can_skip_payment():
            return self.skip_payment()

        response = super(AdyenRedirectView, self).get(request, *args, **kwargs)
        self.perform_action()
        return response

    def can_skip_payment(self):
        """
        A function to determain if we should skip the payment redirect
        """
        return False

    def skip_payment(self):
        raise NotImplementedError("You can't skip payments yet")

    def post(self, request, *args, **kwargs):
        """
        Normaly this post will go to Adyen. But for local development and intergrated testing this
        function can be used so no user action is required to test the handling.
        """
        if settings.ADYEN_ENABLED:
            raise Http404("Cannot handle post when adyen is enabled")

        order = self.get_object()
        params = {
            "authResult": settings.ADYEN_NEXT_STATUS,
            "pspReference": "Local redirect",
            "merchantReference": order.reference,
            "skinCode": settings.ADYEN_SKIN_CODE,
            "paymentMethod": "ideal",
            "shopperLocale": "NL",
        }

        signed_params = sign_params(params)
        return HttpResponseRedirect("{}?{}".format(self.get_next_url(), urlencode(signed_params)))

    def perform_action(self):
        pass

    def get_next_url(self):
        raise NotImplementedError

    def get_current_url(self):
        return "."


class AdyenResponseMixin(object):
    def post(self, request, *args, **kwargs):
        request.GET = request.POST
        return self.get(request, *args, **kwargs)

    def done(self):
        self.order.save()

        return self.render_to_response(self.get_context_data())

    def get(self, request, *args, **kwargs):
        self.payment_response = HPPPaymentResponse.from_data(request.GET)

        # If 'merchant_reference' is not set in the response (would be None in the object)
        # and we would query with this, get_object_or_404 would do a query on _everything_
        # in the database.
        #
        # I would prefer to do a proper sanitizing step in the class. But I'm refactoring a
        # lot already.
        #
        if self.payment_response.merchant_reference is None:
            raise Http404

        try:
            self.order = get_object_or_404(
                self.model, reference=self.payment_response.merchant_reference
            )
        except Exception:
            raise Http404

        #
        # This is a very important step, otherwise payments can be processed twice. By re-using
        # the URL.
        #
        if self.order.status != Status.Created:
            raise Http404

        if not self.payment_response.is_valid():
            calculated_params = self.payment_response.get_adyen_params()
            logger.debug('Order ref: %s | MerchangeSig not correct', self.payment_response.merchant_reference)
            logger.debug(
                'Order ref: %s | Our Msig: %s | Adyen Msig: %s',
                self.payment_response.merchant_reference,
                calculated_params['merchantSig'],
                self.payment_response.merchant_sig
            )
            return self.handle_error()

        logger.info(
            'Order ref: %s | Received Adyen auth result: %s',
            self.payment_response.merchant_reference,
            self.payment_response.auth_result
        )

        self.handle_default()

        if self.payment_response.auth_result == 'ERROR':
            return self.handle_error()

        if self.payment_response.auth_result == 'CANCELLED':
            return self.handle_canceled()

        if self.payment_response.auth_result == 'REFUSED':
            return self.handle_refused()

        if self.payment_response.auth_result == 'PENDING':
            return self.handle_pending()

        if self.payment_response.auth_result == 'AUTHORISED':
            return self.handle_authorised()

        logger.error("Please implement the following authResult: %s", self.payment_response.auth_result)
        return self.handle_pending()

    def handle_authorised(self):
        raise NotImplementedError()

    def handle_pending(self):
        raise NotImplementedError()

    def handle_refused(self):
        raise NotImplementedError()

    def handle_error(self):
        raise NotImplementedError()

    def handle_canceled(self):
        raise NotImplementedError()

    def handle_default(self):
        return



from datetime import timedelta

from django.apps import apps
from django.core.management.base import BaseCommand
from django.utils import timezone

from djadyen import settings
from djadyen.choices import Status
from djadyen.models import AdyenNotification


class Command(BaseCommand):
    help = "Process the adyen notifications that are not processed yet."

    def handle(self, *args, **options):
        order_models = [apps.get_model(model) for model in settings.ADYEN_ORDER_MODELS]

        # Process notifications which have been sent by Adyen.
        for notification in AdyenNotification.objects.filter(is_processed=False):
            notification_data = notification.get_notification_data()
            reference = notification_data.get('merchantReference')

            for order_model in order_models:
                #
                # TOOD: Ugh, okay so we process only 'Pending' orders, this might
                # or might not be correct, I don't understand the adyen state machine
                # well enough.
                #
                # The reason this is done to avoid that an order, which is already processed
                # in the return URL, to be processed again by a notification.
                #
                orders = order_model.objects.filter(reference=reference, status=Status.Pending)

                for order in orders:
                    order.process_notification(notification)

        # After five days of an Order having status 'Pending', move them to 'Error'
        five_days_ago = timezone.now() - timedelta(days=5)
        for order_model in order_models:
            for obj in order_model.objects.filter(
                status=Status.Pending,
                created_on__lte=five_days_ago
            ):
                obj.status = Status.Error
                obj.save()



import json
from datetime import datetime
from uuid import uuid4

from django.core.management import call_command
from django.test.testcases import TestCase

import requests_mock
from freezegun import freeze_time

from djadyen import settings
from djadyen.choices import Status
from djadyen.models import AdyenIssuer, AdyenPaymentOption

from .factories import NotificationFactory, OrderFactory


def json_response(request, context):
    return json.dumps({
        'paymentMethods': [
            {'brandCode': 'mc', 'name': 'MasterCard'},
            {'brandCode': 'visa', 'name': 'VISA'},
            {'brandCode': 'ideal', 'issuers': [
                {'issuerId': '1121', 'name': 'Test Issuer'},
                {'issuerId': '1154', 'name': 'Test Issuer 5'},
                {'issuerId': '1153', 'name': 'Test Issuer 4'},
                {'issuerId': '1152', 'name': 'Test Issuer 3'},
                {'issuerId': '1151', 'name': 'Test Issuer 2'},
                {'issuerId': '1162', 'name': 'Test Issuer Cancelled'},
                {'issuerId': '1161', 'name': 'Test Issuer Pending'},
                {'issuerId': '1160', 'name': 'Test Issuer Refused'},
                {'issuerId': '1159', 'name': 'Test Issuer 10'},
                {'issuerId': '1158', 'name': 'Test Issuer 9'},
                {'issuerId': '1157', 'name': 'Test Issuer 8'},
                {'issuerId': '1156', 'name': 'Test Issuer 7'},
                {'issuerId': '1155', 'name': 'Test Issuer 6'}
            ], 'name': 'iDEAL'}
        ]}, ensure_ascii=False).encode('gbk')


class SyncPaymentMethods(TestCase):
    def test_on_empty_database(self):
        self.assertEqual(AdyenPaymentOption.objects.count(), 0)
        self.assertEqual(AdyenIssuer.objects.count(), 0)

        with self.assertRaises(ValueError):
            call_command('sync_payment_methods')

        # self.assertEqual(AdyenPaymentOption.objects.count(), 3)
        # self.assertEqual(AdyenIssuer.objects.count(), 13)

    def test_on_existing_database(self):
        self.assertEqual(AdyenPaymentOption.objects.count(), 0)
        self.assertEqual(AdyenIssuer.objects.count(), 0)

        with self.assertRaises(ValueError):
            call_command('sync_payment_methods')

        # self.assertEqual(AdyenPaymentOption.objects.count(), 3)
        # self.assertEqual(AdyenIssuer.objects.count(), 13)

        # call_command('sync_payment_methods')

        # self.assertEqual(AdyenPaymentOption.objects.count(), 3)
        # self.assertEqual(AdyenIssuer.objects.count(), 13)

    @requests_mock.mock()
    def test_on_empty_database_mock(self, mock):
        mock.post(
            'https://test.adyen.com/hpp/directory.shtml',
            [
                {"content": json_response, "status_code": 200},
            ],
        )

        call_command('sync_payment_methods')

        self.assertEqual(AdyenPaymentOption.objects.count(), 3)
        self.assertEqual(AdyenIssuer.objects.count(), 13)

    @requests_mock.mock()
    def test_on_existing_database_mock(self, mock):
        mock.post(
            'https://test.adyen.com/hpp/directory.shtml',
            [
                {"content": json_response, "status_code": 200},
                {"content": json_response, "status_code": 200},
            ],
        )
        self.assertEqual(AdyenPaymentOption.objects.count(), 0)
        self.assertEqual(AdyenIssuer.objects.count(), 0)

        call_command('sync_payment_methods')

        self.assertEqual(AdyenPaymentOption.objects.count(), 3)
        self.assertEqual(AdyenIssuer.objects.count(), 13)

        call_command('sync_payment_methods')

        self.assertEqual(AdyenPaymentOption.objects.count(), 3)
        self.assertEqual(AdyenIssuer.objects.count(), 13)


class ProcessNotifications(TestCase):
    def setUp(self):
        super(ProcessNotifications, self).setUp()

        reference = str(uuid4())

        data = {
            'success': 'true',
            'eventCode': 'AUTHORISATION',
            'merchantReference': reference,
            'merchantAccountCode': settings.ADYEN_MERCHANT_ACCOUNT,
        }

        self.notification1 = NotificationFactory.create(
            notification=json.dumps(data),
            is_processed=False
        )
        self.order1 = OrderFactory.create(
            status=Status.Pending,
            reference=reference
        )

    @freeze_time('2019-01-01 12:00')
    def test_process_notifications_already_processed(self):
        """
        Make sure that an order, which status has already been
        set as 'Authorised' is not processed again.
        """
        self.order1.status = Status.Authorised
        self.order1.save()

        self.assertFalse(self.order1.paid)

        call_command('adyen_maintenance')

        self.order1.refresh_from_db()
        self.assertFalse(self.order1.paid)

    @freeze_time('2019-01-01 12:00')
    def test_process_notifications(self):
        self.assertFalse(self.order1.paid)

        call_command('adyen_maintenance')

        self.order1.refresh_from_db()
        self.assertTrue(self.order1.paid)

        self.notification1.refresh_from_db()
        self.assertTrue(self.notification1.is_processed)
        self.assertTrue(self.notification1.processed_at, datetime(2019, 1, 1, 12, 0))


class CleanupPending(TestCase):
    def test_cleanup(self):
        # 5 days ago; Should be marked as 'Error'
        with freeze_time('2019-01-5 12:00'):
            self.order1 = OrderFactory.create(status=Status.Pending)
        # 4 days ago; Should be left alone
        with freeze_time('2019-01-6 12:00'):
            self.order2 = OrderFactory.create(status=Status.Pending)
        # 6 days ago, Should be marked as 'Error'
        with freeze_time('2019-01-4 12:00'):
            self.order3 = OrderFactory.create(status=Status.Pending)
        # 7 days ago, but Authorised, should be left alone
        with freeze_time('2019-01-3 12:00'):
            self.order4 = OrderFactory.create(status=Status.Authorised)

        with freeze_time('2019-01-10 12:00'):
            call_command('adyen_maintenance')

        self.order1.refresh_from_db()
        self.order2.refresh_from_db()
        self.order3.refresh_from_db()
        self.order4.refresh_from_db()

        self.assertEqual(self.order1.status, Status.Error)
        self.assertEqual(self.order2.status, Status.Pending)
        self.assertEqual(self.order3.status, Status.Error)
        self.assertEqual(self.order4.status, Status.Authorised)




import json
from django.urls import reverse

from django_webtest import WebTest

from djadyen.models import AdyenNotification


class NotificationViewTests(WebTest):
    def setUp(self):
        self.url = reverse('adyen-notifications:notification')

        self.post_data = {
            'additionalData.hmacSignature': 'sJPOWBVc8nZRBX8/6xW8hsqyo381D8nsDkVThu++9LU=',
            'merchantAccountCode': 'MaykinMediaNL',
            'live': 'false',
            'originalReference': '',
            'paymentMethod': 'ideal',
            'value': '350',
            'pspReference': '8515535308218301',
            'operations': 'REFUND',
            'eventDate': '2019-03-25T16:20:23.71Z',
            'success': 'true',
            'currency': 'EUR',
            'merchantReference': '2b947321-527c-4b69-9cca-1f278cb4b23b',
            'eventCode': 'AUTHORISATION',
        }

    def test_post_notification_empty_hmac(self):
        """
        An empty hmac signature is always a nice edge case.
        """
        params = {
            'additionalData.hmacSignature': '',
        }
        response = self.app.post(self.url, params=params, status=403)

        self.assertEqual(response.status_code, 403)
        self.assertEqual(AdyenNotification.objects.count(), 0)

    def test_post_notification_no_hmac(self):
        """
        Make sure that a notification without a HMAC set, does not work.
        """
        del self.post_data['additionalData.hmacSignature']
        response = self.app.post(self.url, params=self.post_data, status=403)
        self.assertEqual(response.status_code, 403)
        self.assertEqual(AdyenNotification.objects.count(), 0)

    def test_hmac_verification_invalid_hmac(self):
        self.post_data['additionalData.hmacSignature'] += 'x'
        response = self.app.post(self.url, self.post_data, status=403)
        self.assertEqual(response.status_code, 403)
        self.assertFalse(AdyenNotification.objects.exists())

    def test_hmac_verification(self):
        self.app.post(self.url, params=self.post_data)
        self.assertTrue(AdyenNotification.objects.exists())

        notification = AdyenNotification.objects.get()
        self.assertEqual(json.loads(notification.notification), self.post_data)



from django.db import models
from djadyen.models import AdyenOrder


class Order(AdyenOrder):
    paid = models.BooleanField(default=False)

    def get_price_in_cents(self):
        return 5000

    def process_notification(self, notification):
        super(Order, self).process_notification(notification)
        if notification.is_authorised():
            self.paid = True
            self.save()



from django.conf.urls import url

from .views import (
    Confirmation2View, ConfirmationView, My2AdyenRequestView,
    My3AdyenRequestView, MyAdyenRequestView
)

urlpatterns = [
    url(r'^confirm/$', ConfirmationView.as_view(), name="confirm"),
    url(r'^confirm2/$', Confirmation2View.as_view(), name="confirm2"),
    url(r'^(?P<reference>[\w-]+)/$', MyAdyenRequestView.as_view(), name="redirect"),
    url(r'^2/(?P<reference>[\w-]+)/$', My2AdyenRequestView.as_view(), name="redirect2"),
    url(r'^3/(?P<reference>[\w-]+)/$', My3AdyenRequestView.as_view(), name="redirect3"),
]



from django.urls import reverse
from django.utils import timezone
from django.views.generic import TemplateView

from djadyen.choices import Status
from djadyen.hpp import HPPPaymenRequest
from djadyen.views import AdyenRedirectView, AdyenResponseMixin

from .models import Order


class MyAdyenRequestView(AdyenRedirectView):
    model = Order

    def get_next_url(self):  # This is to populate the resURL
        return reverse('confirm')


class My2AdyenRequestView(AdyenRedirectView):
    model = Order

    def get_payment_request(self, obj):
        payment_request = HPPPaymenRequest.from_object(obj, self.get_next_url())
        payment_request.ship_before_date = timezone.now()
        payment_request.merchant_return_data = 'returnData'
        return payment_request

    def get_next_url(self):
        return ''


class My3AdyenRequestView(AdyenRedirectView):
    model = Order

    def get_payment_request(self, obj):
        payment_request = HPPPaymenRequest.from_object(obj, self.get_next_url())
        payment_request.ship_before_date = timezone.now()
        payment_request.merchant_return_data = 'returnData'
        return payment_request

    def can_skip_payment(self):
        return True


class ConfirmationView(AdyenResponseMixin, TemplateView):
    template_name = 'app/confirmation.html'
    model = Order

    def handle_authorised(self):
        self.order.status = Status.Authorised
        return self.done()

    def handle_pending(self):
        self.order.status = Status.Pending
        return self.done()

    def handle_refused(self):
        self.order.status = Status.Refused
        return self.done()

    def handle_error(self):
        self.order.status = Status.Error
        return self.done()

    def handle_canceled(self):
        self.order.status = Status.Cancel
        return self.done()

    def handle_default(self):
        if self.payment_response.psp_reference:
            self.order.psp_reference = self.payment_response.psp_reference


class Confirmation2View(AdyenResponseMixin, TemplateView):
    template_name = 'app/confirmation.html'
    model = Order



import base64
import binascii
import hashlib
import hmac
from collections import OrderedDict

from djadyen import settings


def escape_val(val):
    try:
        return val.replace('\\', '\\\\').replace(':', '\\:')
    except AttributeError:
        return str(val)


def create_hmac(data):
    """
    Description of how the HMAC can be calculated can be found here

    https://docs.adyen.com/developers/development-resources/notifications/signing-notifications-with-hmac
    """
    hmac_key = binascii.a2b_hex(settings.ADYEN_NOTIFICATION_SECRET)

    sign_fields = [
        'pspReference',
        'originalReference',
        'merchantAccountCode',
        'merchantReference',
        'value',
        'currency',
        'eventCode',
        'success'
    ]

    msg_params = OrderedDict(
        [(key, data.get(key, '')) for key in sign_fields]
    )
    hmac_input = ':'.join([escape_val(value) for key, value in msg_params.items()])
    hmac_value = hmac.new(hmac_key, hmac_input.encode('utf-8'), hashlib.sha256)
    return (
        base64.b64encode(hmac_value.digest()).decode("utf-8")
    )
